import java.io.*;
import java.util.Properties;
import java.util.Date;
import jcifs.util.Hexdump;
import msrpc.*;

public class TestSrvsvc extends srvsvc {

  String servername;

  TestSrvsvc(String servername, Properties properties) {
    this.servername = servername;
    setAddress("ncacn_np:" + servername + "[\\PIPE\\srvsvc]");
    setProperties(properties);
  }

  public void doShareEnum() throws Exception {
    ShareInfoCtr1 info = new ShareInfoCtr1();
    ShareEnumAll req = new ShareEnumAll("\\\\" + servername, 1, info, -1, 0, 0);

    call(0, req);
    if (req.retval != 0) {
      throw new Exception("0x" + Hexdump.toHexString(req.retval, 8));
    }

    for (int i = 0; i < info.count; i++) {
      ShareInfo1 i1 = info.array[i];
      System.out.println("info[" + i + "]: " + i1.netname + " "
          + Hexdump.toHexString(i1.type, 8) + " " + i1.remark);
    }
  }

  public void doServerGetInfo100() throws Exception {
    ServerInfo100 info = new ServerInfo100();
    ServerGetInfo req = new ServerGetInfo("\\\\" + servername, 100, info);

    call(0, req);
    if (req.retval != 0) {
      throw new Exception("0x" + Hexdump.toHexString(req.retval, 8));
    }

    String platformName;
    if (info.platform_id == PLATFORM_ID_NT)
      platformName = "PLATFORM_ID_NT";
    else
      platformName = "UNSUPPORTED";

    System.out.println("platform_id: " + platformName + "(" + info.platform_id
        + ") name: " + info.name);
  }

  public void doServerGetInfo102() throws Exception {
    ServerInfo102 info = new ServerInfo102();
    ServerGetInfo req = new ServerGetInfo("\\\\" + servername, 102, info);

    call(0, req);
    if (req.retval != 0) {
      throw new Exception("0x" + Hexdump.toHexString(req.retval, 8));
    }

    StringBuffer typeSb = new StringBuffer();
    if ((info.type & SV_TYPE_WORKSTATION) != 0)
      typeSb.append("workstation, ");
    if ((info.type & SV_TYPE_SERVER) != 0)
      typeSb.append("server, ");
    if ((info.type & SV_TYPE_SQLSERVER) != 0)
      typeSb.append("sql server, ");
    if ((info.type & SV_TYPE_DOMAIN_CTRL) != 0)
      typeSb.append("domain controller, ");
    if ((info.type & SV_TYPE_NT) != 0)
      typeSb.append("nt workstation, ");
    if ((info.type & SV_TYPE_DOMAIN_BAKCTRL) != 0)
      typeSb.append("backup domain controller, ");
    if ((info.type & SV_TYPE_SERVER_UNIX) != 0)
      typeSb.append("unix server, ");
    if ((info.type & SV_TYPE_SERVER_NT) != 0)
      typeSb.append("nt server, ");
    if ((info.type & SV_TYPE_POTENTIAL_BROWSER) != 0)
      typeSb.append("potential browser, ");
    if ((info.type & SV_TYPE_WINDOWS) != 0)
      typeSb.append("windows 95+, ");

    System.out.println("version: " + info.version_major + "."
        + info.version_minor);
    System.out.println("type: " + typeSb.toString() + "(" + info.type + ")");
  }

  public Date doRemoteTOD() throws Exception {
    TimeOfDayInfo info = new TimeOfDayInfo();
    RemoteTOD req = new RemoteTOD("\\\\" + servername, info);

    call(0, req);
    if (req.retval != 0) {
      throw new Exception("0x" + Hexdump.toHexString(req.retval, 8));
    }

    Date d = new Date(info.elapsedt * 1000L);
    System.out.println("Time: " + d);

    return d;
  }

  public void doAll() throws Exception {
//    doShareEnum();
//    doServerGetInfo100();
    doServerGetInfo102();
//    doRemoteTOD();
  }

  public static void main(String[] args) throws Exception {
    if (args.length < 1) {
      System.err.println("usage: TestSrvsvc <servername> [<properties>]");
      return;
    }

    Properties properties = null;
    if (args.length > 1) {
      properties = new Properties();
      properties.load(new FileInputStream(args[1]));
    }

    TestSrvsvc stub = new TestSrvsvc(args[0], properties);

    stub.doAll();
  }
}
